/* * * * * * * * * * * * * * * * * * * * * * * *
 * SCIDYNE Corporation, May 10, 2022,  Mark Durgin 
 * Simple digital I/O demo program for DIO64-ARD PN #100-7728
 * - Repeatedly writes pattern of alternating 0's and 1's to Port-A of the designated MCP23017 chip
 *   Use an oscilloscope to observe results
 * - Reads state of Port-B and prints to serial monitor. Short pins to ground to cause changes
 */
#include <Wire.h>   // Needed for Arduino I2C support

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Program constants and defines
 */

// Define only one MCP23017 Chip Address, comment-out the other three
#define MCP23017_ADDRESS   0x20       // Chip Address for PortA & PortB
//#define MCP23017_ADDRESS   0x21     // Chip Address for PortC & PortD
//#define MCP23017_ADDRESS   0x22     // Chip Address for PortE & PortF
//#define MCP23017_ADDRESS   0x23     // Chip Address for PortG & PortH

// Define the MCP23017 registers to be used. See the MCP23017 data sheet for details
#define MCP23017_IODIRA       0x00      // MCP23017 Port-A Data Direction Register
#define MCP23017_GPIOA        0x12      // MCP23017 Port-A Data register
#define MCP23017_IODIRB       0x01      // MCP23017 Port-B Data Direction Register
#define MCP23017_GPIOB        0x13      // MCP23017 Port-B Data register
#define MCP23017_GPPUB        0x0D      // MCP23017 Port-B Pull-Up enable register

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * setup runs once then execution goes to loop
 */
void setup()
{
  Wire.begin();                             // Setup I2C communications

  Serial.begin(115200);                       // Setup communications to IDE Monitor
    while (!Serial);

  // Setup MCP23017 Port-A as all outputs
  Wire.beginTransmission(MCP23017_ADDRESS);         // Open I2C communication with the MCP23017
  Wire.write(MCP23017_IODIRA);                      // Next byte intended for Port-A direction register
  Wire.write(0x00);                                 // Make all outputs, Bits set as 0 are outputs
  Wire.endTransmission();                           // Close I2C communications

 // Setup MCP23017 Port-B as all inputs
  Wire.beginTransmission(MCP23017_ADDRESS);         // Open I2C communication with the MCP23017
  Wire.write(MCP23017_IODIRB);                      // Next byte intended for Port-B direction register
  Wire.write(0xFF);                                 // Make all inputs, Bits set as 1 are inputs
  Wire.endTransmission();                           // Close I2C communications

  Wire.beginTransmission(MCP23017_ADDRESS);         // Open I2C communication with the MCP23017
  Wire.write(MCP23017_GPPUB);                       // Next byte intended for Port-B Pull-Up register
  Wire.write(0xFF);                                 // Bits set as 1 enable corresponding pull-up
  Wire.endTransmission();                           // Close I2C communications

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * loop runs continuously
 */
void loop()
{
  // Toggle bit pattern on Port-A
  Wire.beginTransmission(MCP23017_ADDRESS);         // Open I2C communication with the MCP23017
  Wire.write(MCP23017_GPIOA);                       // Next byte intended for Port-A data register
  Wire.write(0xAA);                                 // Make Port-A = 1 0 1 0 1 0 1 0
  Wire.endTransmission();                           // Close I2C communications

  delay(100);                                       // Wait 100ms
  
  Wire.beginTransmission(MCP23017_ADDRESS);         // Open I2C communication with the MCP23017
  Wire.write(MCP23017_GPIOA);                       // Next byte intended for Port-A data register
  Wire.write(0x55);                                 // Make Port-A = 0 1 0 1 0 1 0 1
  Wire.endTransmission();                           // Close I2C communications

  delay(100);                                       // Wait 100ms

  // Read the state of Port-B and display on Arduino IDE Serial Monitor
  Wire.beginTransmission(MCP23017_ADDRESS);         // Open I2C communication with the MCP23017
  Wire.write(MCP23017_GPIOB);                       // Read will be from Port-B data register
  Wire.endTransmission();

  Wire.requestFrom(MCP23017_ADDRESS, 1);            // Request 1 byte from the MCP23017
  unsigned char x = Wire.read();                    // Receive the Port-B byte and store in x
  Serial.println(x);                                // Print the character
  
  delay(100);                                       // Wait 100ms

}
